@@ -2,14 +2,70 @@ use crate::de::{Deserialize, DeserializeError};
 use crate::ser::{Serialize, SerializeError};
 use crate::{SimpleSerialize, Sized};
 use sha2::{Digest, Sha256};
-use std::{cmp::Ordering, convert::TryInto, ops::Index};
+use std::array::TryFromSliceError;
+use std::cmp::Ordering;
+use std::convert::AsRef;
+use std::fmt;
+use std::ops::{Index, IndexMut};
 use thiserror::Error;
 
 pub(crate) const BYTES_PER_CHUNK: usize = 32;
 
-pub(crate) const ZERO_CHUNK: &[u8] = &[0; BYTES_PER_CHUNK];
+#[derive(Default, Clone, Copy, PartialEq, Eq)]
+pub struct Root([u8; 32]);
 
-pub type Root = [u8; 32];
+impl Root {
+    pub fn from_bytes(root: [u8; 32]) -> Self {
+        Self(root)
+    }
+}
+
+impl fmt::LowerHex for Root {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        if f.alternate() {
+            write!(f, "0x")?;
+        }
+        for i in &self.0[..] {
+            write!(f, "{:02x}", i)?;
+        }
+        Ok(())
+    }
+}
+
+impl fmt::Debug for Root {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{:#x}", self)
+    }
+}
+
+impl AsRef<[u8; 32]> for Root {
+    fn as_ref(&self) -> &[u8; 32] {
+        &self.0
+    }
+}
+
+impl Index<usize> for Root {
+    type Output = u8;
+
+    fn index(&self, index: usize) -> &Self::Output {
+        &self.0[index]
+    }
+}
+
+impl IndexMut<usize> for Root {
+    fn index_mut(&mut self, index: usize) -> &mut Self::Output {
+        &mut self.0[index]
+    }
+}
+
+impl TryFrom<&[u8]> for Root {
+    type Error = TryFromSliceError;
+
+    fn try_from(value: &[u8]) -> Result<Self, Self::Error> {
+        let inner = value.try_into()?;
+        Ok(Self(inner))
+    }
+}
 
 impl Sized for Root {
     fn is_variable_size() -> bool {
@@ -23,7 +79,7 @@ impl Sized for Root {
 
 impl Serialize for Root {
     fn serialize(&self, buffer: &mut Vec<u8>) -> Result<usize, SerializeError> {
-        buffer.extend_from_slice(self);
+        buffer.extend_from_slice(self.as_ref());
         Ok(Self::size_hint())
     }
 }
@@ -47,7 +103,7 @@ impl Deserialize for Root {
 
 impl Merkleized for Root {
     fn hash_tree_root(&self, _context: &Context) -> Result<Root, MerkleizationError> {
-        Ok(*self)
+        Ok(Root(self.0))
     }
 }
 
@@ -245,8 +301,13 @@ fn mix_in_decoration(root: &Root, decoration: usize, context: &Context) -> Root
 
     let mut hasher = Sha256::new();
     let mut output = vec![0u8; BYTES_PER_CHUNK];
-    hash_nodes(&mut hasher, root, &decoration_data, &mut output);
-    output.try_into().expect("can extract root")
+    hash_nodes(
+        &mut hasher,
+        root.as_ref(),
+        decoration_data.as_ref(),
+        &mut output,
+    );
+    output.as_slice().try_into().expect("can extract root")
 }
 
 pub(crate) fn mix_in_length(root: &Root, length: usize, context: &Context) -> Root {
@@ -331,10 +392,11 @@ mod tests {
         let mut hasher = Sha256::new();
         let mut buffer = vec![0u8; node_count * BYTES_PER_CHUNK];
         buffer[leaf_start..leaf_start + chunks.len()].copy_from_slice(chunks);
+        let zero_chunk = [0u8; 32];
         for i in chunks.len()..leaf_count {
             let start = leaf_start + (i * BYTES_PER_CHUNK);
             let end = leaf_start + (i + 1) * BYTES_PER_CHUNK;
-            buffer[start..end].copy_from_slice(ZERO_CHUNK);
+            buffer[start..end].copy_from_slice(&zero_chunk);
         }
 
         for i in (1..node_count).rev().step_by(2) {
@@ -356,42 +418,42 @@ mod tests {
         let chunks = vec![0u8; 2 * BYTES_PER_CHUNK];
         let root = merkleize_chunks(&chunks, 2).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("f5a5fd42d16a20302798ef6ed309979b43003d2320d9f0e8ea9831a92759fb4b")
         );
 
         let chunks = vec![1u8; 2 * BYTES_PER_CHUNK];
         let root = merkleize_chunks(&chunks, 2).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("7c8975e1e60a5c8337f28edf8c33c3b180360b7279644a9bc1af3c51e6220bf5")
         );
 
         let chunks = vec![0u8; BYTES_PER_CHUNK];
         let root = merkleize_chunks(&chunks, 4).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("db56114e00fdd4c1f85c892bf35ac9a89289aaecb1ebd0a96cde606a748b5d71")
         );
 
         let chunks = vec![1u8; BYTES_PER_CHUNK];
         let root = merkleize_chunks(&chunks, 4).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("29797eded0e83376b70f2bf034cc0811ae7f1414653b1d720dfd18f74cf13309")
         );
 
         let chunks = vec![2u8; BYTES_PER_CHUNK];
         let root = merkleize_chunks(&chunks, 8).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("fa4cf775712aa8a2fe5dcb5a517d19b2e9effcf58ff311b9fd8e4a7d308e6d00")
         );
 
         let chunks = vec![1u8; 5 * BYTES_PER_CHUNK];
         let root = merkleize_chunks(&chunks, 8).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("0ae67e34cba4ad2bbfea5dc39e6679b444021522d861fab00f05063c54341289")
         );
     }
@@ -404,7 +466,7 @@ mod tests {
         let root =
             merkleize_chunks_with_virtual_padding(&chunks, 4, &context).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("65aa94f2b59e517abd400cab655f42821374e433e41b8fe599f6bb15484adcec")
         );
 
@@ -412,7 +474,7 @@ mod tests {
         let root =
             merkleize_chunks_with_virtual_padding(&chunks, 8, &context).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("0ae67e34cba4ad2bbfea5dc39e6679b444021522d861fab00f05063c54341289")
         );
 
@@ -420,7 +482,7 @@ mod tests {
         let root =
             merkleize_chunks_with_virtual_padding(&chunks, 8, &context).expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("0ef7df63c204ef203d76145627b8083c49aa7c55ebdee2967556f55a4f65a238")
         );
     }
@@ -433,7 +495,7 @@ mod tests {
         let root = merkleize_chunks_with_virtual_padding(&chunks, 2usize.pow(10), &context)
             .expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("2647cb9e26bd83eeb0982814b2ac4d6cc4a65d0d98637f1a73a4c06d3db0e6ce")
         );
 
@@ -441,7 +503,7 @@ mod tests {
         let root = merkleize_chunks_with_virtual_padding(&chunks, 2usize.pow(63), &context)
             .expect("can merkleize");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("9317695d95b5a3b46e976b5a9cbfcfccb600accaddeda9ac867cc9669b862979")
         );
     }
@@ -480,7 +542,7 @@ mod tests {
         ]);
         let root = a_list.hash_tree_root(&context).expect("can compute root");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("d20d2246e1438d88de46f6f41c7b041f92b673845e51f2de93b944bf599e63b1")
         );
     }
@@ -491,7 +553,7 @@ mod tests {
         let a_list = List::<u16, 1024>::from_iter([]);
         let root = a_list.hash_tree_root(&context).expect("can compute root");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("c9eece3e14d3c3db45c38bbf69a4cb7464981e2506d8424a0ba450dad9b9af30")
         );
     }
@@ -539,7 +601,7 @@ mod tests {
 
         let root = foo.hash_tree_root(&context).expect("can make root");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("7078155bf8f0dc42d8afccec8d9b5aeb54f0a2e8e58fcef3e723f6a867232ce7")
         );
 
@@ -553,13 +615,13 @@ mod tests {
             .hash_tree_root(&context)
             .expect("can make root");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("7078155bf8f0dc42d8afccec8d9b5aeb54f0a2e8e58fcef3e723f6a867232ce7")
         );
 
         let root = foo.hash_tree_root(&context).expect("can make root");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("0063bfcfabbca567483a2ee859fcfafb958329489eb328ac7f07790c7df1b231")
         );
 
@@ -571,7 +633,7 @@ mod tests {
             .hash_tree_root(&context)
             .expect("can make root");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("7078155bf8f0dc42d8afccec8d9b5aeb54f0a2e8e58fcef3e723f6a867232ce7")
         );
 
@@ -581,7 +643,7 @@ mod tests {
 
         let root = foo.hash_tree_root(&context).expect("can make root");
         assert_eq!(
-            root,
+            *root.as_ref(),
             hex!("0063bfcfabbca567483a2ee859fcfafb958329489eb328ac7f07790c7df1b231")
         );
     }
